do

--------------------------------------------------------------
-----------Globals
--------------------------------------------------------------
--assert(loadfile("H:\\scripts\\DynamicACM.lua"))()
  
 ACMScriptVersionNumber = "1.0.0" --Version number displayed on load
 SecondsMessagesDisplayed = 30 --How many seconds that report messages stay on screen
 
 MinWaveInterval = 30 --Minimum time (in seconds) between wildcard waves
 MaxWaveInterval = 60 --Maximum time (in seconds) between wildcard waves
 MinWaveAlt = 3000 --Lowest altitude (in meters) that a wildcard bandit will spawn
 MaxWaveAlt = 6000 --Highest altitude (in meters) that a wildcard bandit will spawn


 RedGroups = {} --Table where Reds are stored
 KillTimes = {} --Table where kill timer counts are stored
 --SpawnZone = ZONE:New( "Spawn Zone" ) --Zone where wildcards will spawn


 --ACM Timer Specific Globals

 ACMFightStatus = "Pre-Fight" --Stages of an ACM fight tracked here
 MaximumMergeDistance = 1000 -- meters distant that red and blue can be for it to be considered a valid merge
 TimerCount = 0 --Time in seconds track here
 RedUnitCount = 0 --Var for keeping track of active Red ACM units
 BlueKills = 0 --Var for tracking blue victories
 TimerActive = false --Whether or not we are actively timing a fight (set to true when a timer is desired)
 DebugMode = false --Activates debug messages

 --------------------------------------------------------------
 ----The following need to be added in the ME:
 --"WildcardZone" At least one zone with a name prefixed with this
 --"Bandit" One or more red air groups with this in the name (late activated)
 --Player aircraft must be in the Blue coalition
 --------------------------------------------------------------   

 --------------------------------------------------------------
 --Bandits spawn at height set in ME (Base mission: 13000 MSL)
 --Wildcards spawn between MinWaveAlt and MaxWaveAlt (set above)
 --------------------------------------------------------------
 
 --------------------------------------------------------------
 -----------Templates
 --------------------------------------------------------------
 
 SetBanditGroups = SET_GROUP:New():FilterCoalitions("red"):FilterPrefixes("Bandit"):FilterOnce()
 TemplateTableBandit = {}
   SetBanditGroups:ForEachGroup( 
   function( GroupObject )
    TemplateTableBandit[#TemplateTableBandit + 1] = GroupObject.GroupName    
   end )

  MiG28SetGroup = SET_GROUP:New():FilterCoalitions("red"):FilterPrefixes("MiG-28"):FilterOnce()
  TemplateTableMiG28 = {}
  MiG28SetGroup:ForEachGroup( 
   function( GroupObject )
    TemplateTableMiG28[#TemplateTableMiG28 + 1] = GroupObject.GroupName    
   end )

 SetSpawnZones = SET_ZONE:New():FilterPrefixes( "WildcardZone" ):FilterOnce() 

 --------------------------------------------------------------
 -----------ACM Timer Functions
 --------------------------------------------------------------

 function GetRedUnits()
    Output = 0
    RedUnits = SET_UNIT:New():FilterCoalitions("red"):FilterActive():FilterOnce()
    RedUnits:ForEachUnit(
    function( RUnit ) 
        Output = Output + 1
    end 
    )
    return Output
 end

 function WithinMergeDistance( MergeDistance )
    
    MergeOutput = false
    RedUnits = SET_UNIT:New():FilterCoalitions( "red" ):FilterActive():FilterOnce()
    BlueUnits = SET_UNIT:New():FilterCoalitions( "blue" ):FilterActive():FilterOnce()
    RedUnits:ForEachUnit(
    function( RUnit )           
        BlueUnits:ForEachUnit(
        function( BUnit )
            local InMergeRadius
            InMergeRadius = RUnit:OtherUnitInRadius( BUnit, MergeDistance )
            if InMergeRadius then               
              MergeOutput = true
            end
        end 
        )
    end 
    )        
    return MergeOutput
 end

  EventHandlerKill = EVENTHANDLER:New() -- create an eventhandler
  EventHandlerKill:HandleEvent(EVENTS.Kill) -- subscribe to event "KILL"
  function EventHandlerKill:OnEventKill(EventData) -- execute this function when event occurs
  --MESSAGE:New( EventData.IniDCSGroupName.." got a kill!" , 10):ToAll()  
    if ( EventData.IniCoalition == coalition.side.BLUE and EventData.TgtCoalition == coalition.side.RED ) then
      BlueKills = BlueKills + 1
      if DebugMode then
        MESSAGE:New( "Blue Team's Total Kills This Set: " .. BlueKills, SecondsMessagesDisplayed):ToAll()
      end
    end
  end

  function UpdateKillTimers( LatestTimerCount )
    KillTimes[#KillTimes + 1] = LatestTimerCount --Update the scoreboard

    if #KillTimes > 1 then
      MESSAGE:New( "Times for the last " .. #KillTimes .. " sets", SecondsMessagesDisplayed):ToAll()
      for i = 1, #KillTimes, 1
        do
          MESSAGE:New( i .. ". [ " .. KillTimes[i] .. " ] seconds", SecondsMessagesDisplayed):ToAll()
        end
    end    
  end

  function ACMStatusUpdate(CurrentStatus)

    --Event code was changed in a mid-2022 patch and sometimes kills are counted more than once, this should catch that
    if CurrentStatus == "Post-Fight" then
      BlueKills = 0
    end
      
    if CurrentStatus == "Pre-Fight" then
      if WithinMergeDistance( MaximumMergeDistance ) then
        ACMFightStatus = "Fight's On"
        TimerCount = 0
        MESSAGE:New("Merged. VID Hostile. " .. ACMFightStatus .. "!" , SecondsMessagesDisplayed):ToBlue()		  
      end
    end

    if CurrentStatus == "Fight's On" then
      if RedUnitCount == 0 then
        ACMFightStatus = "Post-Fight"
        MESSAGE:New("Fight lasted " .. TimerCount .. " seconds.", SecondsMessagesDisplayed):Clear():ToBlue() 
        UpdateKillTimers(TimerCount)
        TimerCount = 0
        BlueKills = 0
      end

      if BlueKills >= RedUnitCount and TimerCount > 0 then
        ACMFightStatus = "Post-Fight"
        MESSAGE:New("Fight lasted " .. TimerCount .. " seconds.", SecondsMessagesDisplayed):Clear():ToBlue()        
        UpdateKillTimers(TimerCount)
        TimerCount = 0
        BlueKills = 0
      end
    end
  
  end

  --This is the scheduler that powers the "ACM Timer"
  ACMScheduler = SCHEDULER:New( nil,
  function()	
   if TimerActive then 
      TimerCount = TimerCount + 1           
      RedUnitCount = GetRedUnits()
      ACMStatusUpdate(ACMFightStatus)
      --Debug messages
      if DebugMode then
        MessageAll = MESSAGE:New( "Timer Count: " .. TimerCount, 1):ToAll() 
        MessageAll = MESSAGE:New( "Red Unit Count: " .. RedUnitCount, 1):ToAll()
        MessageAll = MESSAGE:New( "Blue Kill Count: " .. BlueKills, 1):ToAll()
        MessageAll = MESSAGE:New( "Status: " .. ACMFightStatus, 1):ToAll()
        MergeMsg = WithinMergeDistance( 1000 )
        MessageAll = MESSAGE:New( "Merged: " .. tostring(MergeMsg), 1):ToAll()
      end
   end
  end, 
  {}, 0, 1 )

 --------------------------------------------------------------
 -----------Spawn Functions
 --------------------------------------------------------------


    function SpawnBanditGroup( BanditName )
        if RedUnitCount == 0 then
          if ACMFightStatus == "Wildcard" then
            --do nothing; a wildcard fight is happening
          else            
            RedGroups[#RedGroups + 1] = SPAWN:New( BanditName ) --Bandit name goes here        
            :Spawn()
            TimerActive = true --start the clock (and tracking)
            ACMFightStatus = "Pre-Fight"
          end
        end
    end

    --Main spawn object for wildcards    
    WildCardSpawn = SPAWN:New( "Wildcard" )

    function SpawnBanditAndWildcards( SpawnType )      

      if RedUnitCount == 0 then        
        ACMFightStatus = "Wildcard"
        TimerActive = false --stop the clock
        local SpawnZoneZero = SetSpawnZones:GetRandomZone(1)
        local SpawnZoneFirst = SetSpawnZones:GetRandomZone(1)
        local SpawnZoneSecond = SetSpawnZones:GetRandomZone(1)
        local SpawnZoneThird = SetSpawnZones:GetRandomZone(1)

        local WildcardTemplateTable

        if SpawnType == "Random" then
          WildcardTemplateTable = TemplateTableBandit
        end

        if SpawnType == "MiG28" then
          WildcardTemplateTable = TemplateTableMiG28
        end
        
        RedGroups[#RedGroups + 1] = WildCardSpawn:InitRandomizeTemplate( WildcardTemplateTable ):SpawnInZone( SpawnZoneZero, true, MinWaveAlt, MaxWaveAlt )

        local FirstWaveInterval = math.random( MinWaveInterval, MaxWaveInterval )
        local SecondWaveInterval = FirstWaveInterval + math.random( MinWaveInterval, MaxWaveInterval )
        local ThirdWaveInterval = SecondWaveInterval + math.random( MinWaveInterval, MaxWaveInterval )
        local NumWildcards = math.random( 1, 3 )
        if DebugMode then
          MESSAGE:New( "Wildcards to spawn: " .. NumWildcards, 15):ToAll()
        end

        local FirstWildcardScheduler = SCHEDULER:New( nil, 
        function()
          RedGroups[#RedGroups + 1] = WildCardSpawn:InitRandomizeTemplate( WildcardTemplateTable ):SpawnInZone( SpawnZoneFirst, true, MinWaveAlt, MaxWaveAlt ) --Spawn a wildcard
          if DebugMode then
            MESSAGE:New( "1st Wildcard Spawned", 15):ToAll()	
          end
        end
        , { }, FirstWaveInterval )

        --Spawn second wildcard if needed
        if NumWildcards >= 2 then
          local SecondWildcardScheduler = SCHEDULER:New( nil, 
          function()
            RedGroups[#RedGroups + 1] = WildCardSpawn:InitRandomizeTemplate( WildcardTemplateTable ):SpawnInZone( SpawnZoneSecond, true, MinWaveAlt, MaxWaveAlt ) --Spawn a wildcard
            if DebugMode then
              MESSAGE:New( "2nd Wildcard Spawned", 15):ToAll()	
            end
          end
          , { }, SecondWaveInterval )
        end

        --Spawn third wildcard if needed
        if NumWildcards >= 3 then
          local ThirdWildcardScheduler = SCHEDULER:New( nil, 
          function()
            RedGroups[#RedGroups + 1] = WildCardSpawn:InitRandomizeTemplate( WildcardTemplateTable ):SpawnInZone( SpawnZoneThird, true, MinWaveAlt, MaxWaveAlt ) --Spawn a wildcard
            if DebugMode then
              MESSAGE:New( "3rd Wildcard Spawned", 15):ToAll()	
            end
          end
          , { }, ThirdWaveInterval )
        end


        --This is the scheduler that powers the "All Clear" after the last wildcard is defeated
        AllClearScheduler = SCHEDULER:New( nil,
        function()	
          if BlueKills > NumWildcards and ACMFightStatus == "Wildcard" then  --The number of Bandits will be NumWildCards + 1, so when BlueKills gets the last wildcard this should trigger
            MessageAll = MESSAGE:New( "All Clear! All wildcards are splashed.", SecondsMessagesDisplayed):ToAll()
            BlueKills = 0 --Resetting BlueKills should stop the message from being spammed
            ACMFightStatus = "Post-Fight"

            --Debug messages
            if DebugMode then
              MessageAll = MESSAGE:New( "Kills/Bandits: " .. BlueKills .. "/" .. NumWildcards + 1, 10):ToAll()
            end
          end
          
        end, 
        {}, 10, 5 )

      end --end big if for "Wildcard" check

    end

 --------------------------------------------------------------
 -----------Menus and general setup
 --------------------------------------------------------------
 
  local function SetupGroupMenus()
       
    SetMenuGroup = SET_GROUP:New():FilterCoalitions("blue"):FilterActive():FilterOnce()
    --Menus are set up by group so a per group argument can be added to each menu. Then messages can be tailored to only show for that group and not everyone on the server
        SetMenuGroup:ForEachGroup(
            function( AGroup )  
                --Menu option for wildcard setups
                MENU_GROUP_COMMAND:New( AGroup, "Wildcard Random Spawn ", nil, SpawnBanditAndWildcards, "Random" )
                MENU_GROUP_COMMAND:New( AGroup, "Wildcard MiG-28 Spawn ", nil, SpawnBanditAndWildcards, "MiG28" )
                --Menu options for individual spawns
                for i = 1, #TemplateTableBandit, 1
                do                   
                    MENU_GROUP_COMMAND:New( AGroup, "Spawn " .. TemplateTableBandit[i], nil, SpawnBanditGroup, TemplateTableBandit[i] )
                end
        end)
    end
    
  local function ShowVersionNumber( MessageGroup )    
    MESSAGE:New( "[Dynamic ACM]: Version number " .. ACMScriptVersionNumber .. " loaded." , 2):ToGroup( MessageGroup )
  end
  

--Setup menus when a player enters an aircraft. Multiplayer safe version
local BlueFighters = SET_GROUP:New():FilterCategoryAirplane():FilterCoalitions("blue"):FilterPrefixes("Fighter"):FilterActive():FilterStart()
BlueFighters:HandleEvent( EVENTS.PlayerEnterAircraft )
function BlueFighters:OnEventPlayerEnterAircraft( EventData )  
  SetupGroupMenus()
  ShowVersionNumber(EventData.IniGroup)
end  

--[[
local DebugScheduler = SCHEDULER:New( JSUnit, 
    function()
      BlueFighters:ForEachGroup(
        function( BGroup )       
        MESSAGE:New( "Bluefighters PlayerName: " .. BGroup:GetPlayerName() , 25):ToAll() 
        MESSAGE:New( "Bluefighters Count: " .. BGroup:GetPlayerCount() , 25):ToAll()
        MESSAGE:New( "Bluefighters GroupName: " .. BGroup:GetName() , 25):ToAll() 
        MESSAGE:New( "Bluefighters UnitName: " .. BGroup:GetFirstUnitAlive():GetName() , 25):ToAll()
        --MESSAGE:New( "Bluefighters ClientUnitName: " .. BGroup:GetClient():GetClientGroupUnit():GetName() , 25):ToAll() 
      end)      
    end
    , { }, 1, 2 ) -- { Parameters }, Seconds after first run, Seconds between subsequent runs, Randomness factor
]]


--[[
  --The following only works in single player
--Setup menus whenever a player enters an aircraft
Pilot1 = CLIENT:FindByName( "Pilot 1" )
Pilot2 = CLIENT:FindByName( "Pilot 2" )
-- Subscribe to the event. The event occurs when a player enters a slot.
Pilot1:HandleEvent( EVENTS.PlayerEnterUnit )
Pilot2:HandleEvent( EVENTS.PlayerEnterUnit )
function Pilot1:OnEventPlayerEnterUnit( EventData )        
  MESSAGE:New( "Setting up radios menus...", 15):ToClient( Pilot1 )	  
  SetupGroupMenus()
end  

function Pilot2:OnEventPlayerEnterUnit( EventData )      
  MESSAGE:New( "Setting up radios menus...", 15):ToClient( Pilot2 )	  
  SetupGroupMenus()	
end
]]

end